@@ -209,32 +209,14 @@ void dns_random_init(const string& data __attribute__((unused)), bool force) {
 #endif
 }
 
-/* Parts of this code come from arc4random_uniform */
 uint32_t dns_random(uint32_t upper_bound) {
   if (chosen_rng == RNG_UNINITIALIZED)
     dns_random_setup();
 
-  unsigned int min;
   if (upper_bound < 2)
     return 0;
-  /* To avoid "modulo bias" for some methods, calculate
-     minimum acceptable value for random number to improve
-     uniformity.
 
-     On applicable rngs, we loop until the rng spews out
-     value larger than min, and then take modulo out of that.
-  */
-#if (ULONG_MAX > 0xffffffffUL)
-  min = 0x100000000UL % upper_bound;
-#else
-  /* Calculate (2**32 % upper_bound) avoiding 64-bit math */
-  if (upper_bound > 0x80000000)
-    min = 1 + ~upper_bound; /* 2**32 - upper_bound */
-  else {
-    /* (2**32 - (x * 2)) % x == 2**32 % x when x <= 2**31 */
-    min = ((0xffffffff - (upper_bound * 2)) + 1) % upper_bound;
-  }
-#endif
+  unsigned int min = pdns::random_minimum_acceptable_value(upper_bound);
 
   switch(chosen_rng) {
   case RNG_UNINITIALIZED:
